<?php declare(strict_types=1);  
    namespace PmWikiInfo; # add suffix 'new' for development version 
	if (!defined('PmWiki')) exit();
    $PmWikiInfoNew = ''; # empty string for production version, 'new' for development version
    /* PmWikiInfo
+  
<describe purpose of recipe>
See https://www.pmwiki.org/wiki/Cookbook/PmWikiInfo and https://kiwiwiki.nz/pmwiki/pmwiki.php/Cookbook/PmWikiInfo
+
  Copyright 2022-present Simon Davis
  This software is written for PmWiki; you can redistribute it and/or modify it under the terms
  of the GNU General Public License as published by the Free Software Foundation; 
  either version 3 of the License, or (at your option) any later version. 
  See pmwiki.php for full details and lack of warranty.
  +
  Revision History <reverse chronological order please>
  # 2024-05-03 convert to extension
  # 2024-04-26 PHP 8 warnings
  # 2023-10-01 add more variables, refactor out dmsg
  # 2022-12-30 refactor from SimpleRecipeTemplate display more variables, hide server username
  +
*/
//=================================================================================================
#
// Initialise constants
    const NL = "\n";
    const BR = '<br/>' . NL;
    const PMWIKIINFONAME = 'PmWikiInfo';
#
# set debug flag
    $MyDefaultValues ['debugPmWikiInfo'] = false;
	$MyDefaultValues ['stylesPmWikiInfo'] = '';
    $conf = extGetConfig($MyDefaultValues);
    $PmWikiInfo_debugon = boolval ($conf['debugPmWikiInfo']); # if on writes input and output to web page
    ##\SDV($PmWikiInfoDebug, true); # set default debug setting if not defined in an configuration file
    ##$PmWikiInfo_debugon = boolval ($PmWikiInfoDebug); # if on writes input and output to web page
#
	if (!function_exists('dmsg')) {
        if (!isset ($MessagesFmt[__NAMESPACE__])) $MessagesFmt[__NAMESPACE__] = []; # initialise
        function dmsg (string $smsgprefix, $smsgdata) { # local instance
            global $MessagesFmt;
            $MessagesFmt[__NAMESPACE__] [] = 
                $smsgprefix . '= ' . (is_array ($smsgdata)
				    ? @implode (BR . $smsgprefix . '= ', \PHSC ($smsgdata))
					: \PHSC ($smsgdata));
        }
    } # end if
# Version date
    $RecipeInfo[PMWIKIINFONAME] ['Version'] = '2024-05-03' . $PmWikiInfoNew; # PmWiki version numbering is done by date
# recipe version page variable
    $FmtPV['$PmWikiInfoVersion']   = "'" . __NAMESPACE__ . " version {$RecipeInfo[PMWIKIINFONAME] ['Version']}'"; // return version as a custom page variable
# set default formatting for recipe classes
    if (empty ($conf ['stylesPmWikiInfo'])) {
		 $HTMLStylesFmt[__NAMESPACE__] = NL . '.pmwikiinfo {font-size: smaller; font-family: monospace;}' . NL;
	}
	else {
		$HTMLStylesFmt[__NAMESPACE__] = conf ['stylesPmWikiInfo'];
	}
    ##\SDV($HTMLStylesFmt[__NAMESPACE__],
    ##    NL . '.pmwikiinfo {font-size: smaller; font-family: monospace;}' . NL);
#
    if ($PmWikiInfo_debugon) {
		dmsg('<hr>' . __FILE__, $RecipeInfo[PMWIKIINFONAME]['Version']);
        dmsg ('conf[]', $conf);
    }
# declare $PmWikiInfo for (:if enabled PmWikiInfo:) recipe installation check
    $PmWikiInfo = true; # enabled
	$EnablePmWikiInfo = true; # for Extension management
#

## Add a PmWiki custom markup 
    # (:pmwikiinfo Optional parameters:)
    $pmwiki_info_markup_pattern = '/\\(:'
      . mb_strtolower(PMWIKIINFONAME) 
      . '(.*)?:\\)/i';
    # when has to be at least fulltext
    \Markup(PMWIKIINFONAME . $PmWikiInfoNew, #name
      'fulltext', # when, e.g. fulltext, directives
      $pmwiki_info_markup_pattern, # pattern
      __NAMESPACE__ . '\PmWikiInfo_Directive' );
    if ($PmWikiInfo_debugon) dmsg('PmWiki Info markup', $pmwiki_info_markup_pattern);
//
    return; # completed PmWiki Info extension setup
/*-----------------------------------------------------------------------------------------------------------*/
#
/** PmWiki Info parser
*   /param   arguments as documented above
*   /return  The PmWiki-formatted information.
*/
function PmWikiInfo_Directive (array $m):string { 
#
    global $PmWikiInfo_debugon; # import variables
    global $RecipeInfo, $HandleActions, $Conditions, $CustomSyntax, $FmtPV, $FmtV, $HandleAuth, $MarkupExpr;
    global $HTMLStylesFmt, $HTMLHeaderFmt, $HTMLFooterFmt, $IMap, $LinkFunctions, $ROEPAtterns, $ROSPAtterns; 
    global $UploadBlacklist, $UploadExts, $UploadExtSize, $WikiStyle;
    global $ThumbList, $_SERVER;
    # determine login auth level
    $pagename = $GLOBALS['MarkupToHTML']['pagename'];
    $aladmin = false;
    $aledit  = false;
    $alread  = false;
    switch (PmWikiAuthLevel($pagename)) {
        case 'admin' : $aladmin = true;
        case 'edit'  : $aledit  = true;
        case 'read'  : $alread  = true;
        default: break;
    }
    $UploadInfo = [];
    $args = \ParseArgs($m[1]); # contains all text within directive
// 
    if ($PmWikiInfo_debugon) { # display inputs and outputs to wiki page
        dmsg ('<hr>', PMWIKIINFONAME);
        dmsg ('m[]', $m);
        dmsg ('args[]', $args);
    } # end PmWikiInfo_debugon
    $displayopts = array_key_exists ('display', $args)  ? explode(',', $args ['display']) : ['none']; # defaults to none
    $debugopt    = array_key_exists ('debug', $args)    ? $args['debug'] === 'true' : false;
    if ($debugopt) $PmWikiInfo_debugon = true; # set on
    $retVal = '<:block>' . NL; # break out of paragraph
    foreach ($displayopts as $argval) {
        switch (true) {
            case ($argval == 'none'): # none specified
                $retVal .= '!!! PmWiki Info Extension parameters' . NL 
                    . '<span class="pmwikiinfo">'
                    . '<strong>[=(:pmwikiinfo display= :)=]</strong>' . BR
                    . '<strong>display=</strong> all, recipeinfo, conditions, customsyntax, defined, fmtpv, fmtv, '
                    . 'handleactions, handleauth, htmlstylesfmt, htmlheaderfmt, htmlfooterfmt, '
                    . 'imap, inputlabelfmt, linkfunctions, markupexpr, namepattern, pagenamechars, phpinfo, '
                    . 'roepatterns, rospatterns, searchpatterns, server, skins, '
                    . 'thumblist, trailfmt, uploadblacklist, uploadexts, uploadextsize, uploadnamechars, vars, wikistyle, debug' . BR
                    . '<strong>debug=</strong> false, true' 
                    . '</span>' . BR;
                break;
            case ($argval == 'all' || $argval == 'recipeinfo'):
                $retVal .= '!! RecipeInfo' . NL;
                $retVal .= PMWIDisplayInfo ($RecipeInfo);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'conditions'):
                $retVal .= '!! Conditions' . NL;
                $retVal .= PMWIDisplayInfo ($Conditions);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'customsyntax'):
                $retVal .= '!! CustomSyntax' . NL;
                $retVal .= PMWIDisplayInfo ($CustomSyntax);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'fmtpv'):
                $retVal .= '!! FmtPV' . NL;
                $retVal .= PMWIDisplayInfo ($FmtPV, true);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'fmtv'):
                $retVal .= '!! FmtV' . NL;
                $retVal .= PMWIDisplayInfo ($FmtV);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'handleactions'):
                $retVal .= '!! HandleActions' . NL;
                $retVal .= PMWIDisplayInfo ($HandleActions);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'handleauth'):
                $retVal .= '!! HandleAuth' . NL;
                $retVal .= PMWIDisplayInfo ($HandleAuth);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'htmlstylesfmt'):
                $retVal .= '!! HTMLStylesFmt' . NL;
                $retVal .= PMWIDisplayInfo ($HTMLStylesFmt);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'imap'):
                $retVal .= '!! IMap' . NL;
                $retVal .= PMWIDisplayInfo ($IMap);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'linkfunctions'):
                $retVal .= '!! LinkFunctions' . NL;
                $retVal .= PMWIDisplayInfo ($LinkFunctions);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'markupexpr'):
                $retVal .= '!! MarkupExpr' . NL;
                $retVal .= PMWIDisplayInfo ($MarkupExpr);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'htmlheaderfmt'):
                $retVal .= '!! HTMLHeaderFmt' . NL;
                $retVal .= PMWIDisplayInfo ($HTMLHeaderFmt);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'htmlfooterfmt'):
                $retVal .= '!! HTMLFooterFmt' . NL;
                $retVal .= PMWIDisplayInfo ($HTMLFooterFmt);
                if ($argval != 'all') break;
            case ($argval == 'phpinfo'):  # not included in 'all'
                $retVal .= '!! PhpInfo' . NL;
                if ($aladmin) {
                    $retVal .= '(:details summary="PhpInfo":)' .NL;
                    foreach (get_phpinfo() as $info) {
                        $retVal .= '!!! ' . key ($info) . NL;
                        $retVal .= PMWIDisplayInfo ($info);
                    }
                    $retVal .= '(:detailsend:)' . NL;
                } else {$retVal .= 'Please login as "admin".' . BR;}
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'uploadblacklist'):
                $retVal .= '!! UploadBlacklist' . NL;
                $retVal .= PMWIDisplayInfo ($UploadBlacklist);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'uploadexts' || $argval == 'uploadextsize'):
                foreach ($UploadExts as $kext => $kval) $UploadInfo [$kext] ['ext'] = $kval;
                foreach ($UploadExtSize as $kext => $kval) $UploadInfo [$kext] ['size'] = $kval;
                $retVal .= '!! $UploadExts' . NL;
                $retVal .= PMWIDisplayInfo ($UploadInfo, true);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'roepatterns'):
                $retVal .= '!! ROE patterns' . NL;
				if (empty ($ROEPatterns)) $ROEPatterns = null;
                $retVal .= PMWIDisplayInfo ($ROEPatterns);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'rospatterns'):
                $retVal .= '!! ROS patterns' . NL;
				if (empty ($ROSPatterns)) $ROSPatterns = null;
                $retVal .= PMWIDisplayInfo ($ROSPatterns);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'server'):
                $retVal .= '!! $_Server' . NL;
                if ($aladmin) {
                    $retVal .= PMWIDisplayInfo ($_SERVER, true);
                } else {$retVal .= 'Please login as "admin".' . BR;}
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'skins'):
                $retVal .= '!! Skins' . NL;
                $retVal .= PMWISkinList ();
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'thumblist'):
                $retVal .= '!! ThumbList' . NL;
                $retVal .= PMWIDisplayInfo ($ThumbList, true);
                if ($argval != 'all') break;
            case ($argval == 'all' || $argval == 'vars'):
                $retVal .= '!! Variables' . NL;
                $retVal .= PMWIDisplayInfo (PMWIVars ());
                if ($argval != 'all') break;
             case ($argval == 'all' || $argval == 'wikistyle'):
                $retVal .= '!! WikiStyle' . NL;
                $retVal .= PMWIDisplayInfo ($WikiStyle, true);
                if ($argval != 'all') break;
            case ($argval == 'defined'): # not included in 'all'
                $retVal .= '!! Defined' . NL;
                if ($aladmin) {
                    $retVal .= '!!! Defined constants' . NL;
                    $retVal .= PMWIDisplayInfo (get_defined_constants(), true);
                    //$retVal .= PMWIDisplayInfo (array_slice (get_defined_vars(), 0, 100), true);
                    $retVal .= 'Too large to display' . NL;
                    $retVal .= '!! Defined functions' . NL;
                    $retVal .= PMWIDisplayInfo (get_defined_functions(), true);
                } else {$retVal .= 'Please login as "admin".' . BR;}
                if ($argval != 'all') break;
            ##
            case ($argval == 'all' || $argval == 'debug'):
                $retVal .= '!! Debug information' . NL;
                $retVal .= PMWIDebugInfo ($m, $args);
                if ($argval != 'all') break;
            ##
/*          ## disabled as displays too much data
/*          case ($argval == 'all' || $argval == 'entities'):
                $Entities = get_html_translation_table(HTML_ENTITIES);
                $retVal .= '!! HTML entities' . NL;
                $retVal .= PMWIDisplayInfo (array_values ($Entities)); 
                if ($argval != 'all') break;*/
            case ($argval == 'all'): break;
            default: $retVal .= 'Unknown display option: "' . strval ($argval) . '"' . BR;  
        }
    }
    # returns wiki markup
    return $retVal;
} # end PmWikiInfo_Directive
/*-----------------------------------------------------------------------------------------------------------*/
##
// these are functional parts of the extension
# display the information for the array supplied
function PMWIDisplayInfo ($srinfo, bool $details = false): string {
    global $PmWikiInfo_debugon; # import variables
    $retVal = ''; # initialise
    # sort keys (i.e. recipe name) ascending, case insensitive
    if (empty ($srinfo)) return 'No data' . BR;
    if (!ksort ($srinfo, SORT_FLAG_CASE | SORT_STRING)) return 'Sort failed' . BR;
    if ($details) $retVal .= '(:details:)';
    $retVal .= NL . '(:table class="pmwikiinfo":)'; # can't be simple table as some strings have newlines in them
    foreach ($srinfo as $key => $value) {
        $retVal .= NL . '(:cellnr:)' . NL . \Keep (\PHSC (strval ($key), ENT_NOQUOTES)) . NL . '(:cell:)=';
        if (is_array ($value)) {
            $retVal .= NL . '(:cell:)' . NL;
            foreach ($value as $rcpkey => $rcpval) {
                $retVal .= \Keep (\PHSC (strval ($rcpkey))) . ':&nbsp;';
                if (is_array ($rcpval)) {
                    $retVal .= \Keep (\PHSC (print_r($rcpval, true), ENT_NOQUOTES));
                } else {
                    $retVal .= \Keep (\PHSC (strval ($rcpval), ENT_NOQUOTES));
                }
            $retVal .= BR;
            }
        } else {
            $retVal .= NL . '(:cell:)' . NL . \Keep (\PHSC (strval ($value), ENT_NOQUOTES));
        }
    }
    $retVal .= NL . '(:tableend:)' . NL;
    if ($details) $retVal .= '(:detailsend:)' . NL;
    return $retVal;
} # end PMWIDisplayInfo
##
function PmWikiAuthLevel(string $pagename): string {
  global $AuthLevels;
  \SDV($AuthLevels, array('admin', 'edit', 'read'));
  foreach($AuthLevels as $level)
      if (RetrieveAuthPage($pagename, $level, false, READPAGE_CURRENT))
          return $level;
  return '(none)'; # no auth level found
}
// $FmtPV['$AuthLevel'] = 'PmWikiAuthLevel($pn)';
#
function PMWIVars (): array {
    global $ActionSkin, $Author, $AuthorGroup, $CategoryGroup, $CookiePrefix;
    global $DefaultGroup, $DefaultName, $DefaultPage, $GroupHeaderFmt, $GroupFooterFmt, $GUIButtons;
    global $HandleImageTplFmt, $HTMLTagAttr, $InputLabelFmt, $MetaRobots, $NamePattern, $PageListSortComp, $PageNameChars, $RecipeInfo;
    global $SearchPatterns, $SiteAdminGroup, $SiteGroup, $Skin, $EnableAutoSkinList, $PageSkinList, $SkinTemplateIncludeLevel; $TrailFmt;
    global $UploadDir, $UploadDirQuota, $UploadMaxSize, $UploadNameChars, $UploadPrefixFmt, $UploadPrefixQuota, $UploadUrlFmt;
    global $Version, $WikiTitle;
	if (empty ($TrailFmt)) $TrailFmt = '';
    return array (
        'ActionSkin' => $ActionSkin, 
        'Author' => $Author,
        'AuthorGroup' => $AuthorGroup,
        'CategoryGroup' => $CategoryGroup,
        'CookiePrefix' => $CookiePrefix, 
        'DefaultGroup' => $DefaultGroup,
        'DefaultName' => $DefaultName,
        'DefaultPage' => $DefaultPage,
        'GroupFooterFmt' => $GroupFooterFmt,
        'GroupHeaderFmt' => $GroupHeaderFmt,
        'GUIButtons' => $GUIButtons,
        'HandleImageTplFmt' => $HandleImageTplFmt,
        'HTMLTagAttr' => $HTMLTagAttr,
        'InputLabelFmt' => $InputLabelFmt,
        'MetaRobots' => $MetaRobots, 
        'NamePattern' => $NamePattern,
        'PageNameChars' => $PageNameChars,
        'SearchPatterns' => $SearchPatterns,
        'Skin' => $Skin,
        'EnableAutoSkinList' => $EnableAutoSkinList, 
        'PageSkinList' => $PageSkinList, 
        'SkinTemplateIncludeLevel' => $SkinTemplateIncludeLevel,
        'SiteAdminGroup' => $SiteAdminGroup,
        'SiteGroup' => $SiteGroup,
        'TrailFmt' => $TrailFmt,
        'UploadDir' => $UploadDir,
        'UploadDirQuota' => $UploadDirQuota,
        'UploadMaxSize' => $UploadMaxSize,
        'UploadNameChars' => $UploadNameChars,
        'UploadPrefixFmt' => $UploadPrefixFmt,
        'UploadPrefixQuota' => $UploadPrefixQuota,
        'UploadUrlFmt' => $UploadUrlFmt,
        'Version' => $Version,
        'WikiTitle' => $WikiTitle,
        'RecipeInfo' => $RecipeInfo[PMWIKIINFONAME]['Version']
    );
}
##
# from https://www.pmwiki.org/wiki/Cookbook/SkinList
function PMWISkinList() {
  global $FarmD, $PageSkinList, $Skin;

  $skinlist = (array)$PageSkinList;
  foreach (array("pub/skins", "$FarmD/pub/skins") as $skindir) {
     $dirhandle = @opendir($skindir);
     if (!$dirhandle) continue;
     while (($skin = readdir($dirhandle)) !== false) {
       if ($skin[0] == '.') continue;
       if (is_dir("$skindir/$skin")) {
		   if (empty ($skinlist[$skin])) {
			   $skinlist[$skin] = 1;
		   } else {
	           $skinlist[$skin]++;
		   }
	   }
     }
     closedir($dirhandle);
  }
  ksort($skinlist);
  $retVal = '(:div class="pmwikiinfo":)' .NL;
  $retVal .= "[[{\$FullName}?setskin= | ''default skin'' ($Skin)]]" . NL;
  foreach($skinlist as $skin => $cnt) 
    if ($cnt > 0) $retVal .= "&nbsp; [[{\$FullName}?setskin=$skin | $skin]]" . NL;
  return $retVal . '(:divend:)' . BR;
} # end PMWISkinList
##
function PMWIDebugInfo (array $srmarkup, array $srargs): string {
    $retVal = '(:div class="pmwikiinfo":)m[]: ';
    $retVal .= PMWIDispArray ($srmarkup);
    $retVal .= 'args[]: ';
    unset ($srargs ['#']); # don't display this
    $retVal .= PMWIDispArray ($srargs) . NL;
    ob_start();
    $retVal .= ob_get_flush() . NL;
 
    return $retVal . BR . '(:divend:)' . BR;
} # end PMWIDebugInfo
##
function PMWIDispArray (array $ArrayVal):string {
    array_walk_recursive ($ArrayVal, __NAMESPACE__ . '\PMWIsanitise');
    $keeptxt = '';
    foreach ($ArrayVal as $key => $value) {
        $keeptxt .= '[' . $key . '] => ';
        if (is_array ($value)) {
            foreach ($value as $k2 => $v2) $keeptxt .=  $v2 . '; ';
        } else {
            $keeptxt .= $value;
        }
        $keeptxt .= BR;
    }
    # use \Keep as markup contains PmWiki markup
    return \PHSC (\Keep($keeptxt));
}
##
##
    function PMWIsanitise(&$item, $itemkey) {
    # escape html special characters   
        if (is_string($item)) $item = \PHSC($item, ENT_NOQUOTES); 
        return;
    } # end PMWIsanitise
#
// get php information
function get_phpinfo (): array {
    ob_start ();                              // Capturing null, 0,  PHP_OUTPUT_HANDLER_CLEANABLE ^ PHP_OUTPUT_HANDLER_REMOVABLE 
    phpinfo ();                               // phpinfo ()
    $html = trim (ob_get_clean ());           // output and clean
    # obfuscate the following variables
    if (isset($_SERVER['AUTH_USER']))     $html = str_replace($_SERVER['AUTH_USER'], '####', $html);
    if (isset($_SERVER['AUTH_PASSWORD'])) $html = str_replace($_SERVER['AUTH_PASSWORD'], '######', $html);
    if (isset($_SERVER['PHP_AUTH_USER'])) $html = str_replace($_SERVER['PHP_AUTH_USER'], '****', $html);
    if (isset($_SERVER['PHP_AUTH_PW']))   $html = str_replace($_SERVER['PHP_AUTH_PW'], '******', $html);
    $strt = strpos($html, '<body>') + strlen('<body>'); # Only works when <body> has no attributes
    $fnsh = '</body>';
    $html = trim(substr($html, $strt, strpos($html, $fnsh) - $strt));
    $phpinfo = explode (NL, strip_tags($html, '<tr><td><h2>'));
    $retVal = array(); # initialise
    $cat = 'General';
    foreach($phpinfo as $line) {
        // new cat?
        preg_match('~<h2>(.*)</h2>~', $line, $title) ? $cat = $title[1] : null;
        if (preg_match('~<tr><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td></tr>~', $line, $val)) {
            $retVal[$cat][$val[1]] = $val[2];
        }
        elseif (preg_match('~<tr><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td></tr>~', $line, $val)) {
            $retVal[$cat][$val[1]] = array('local' => $val[2], 'master' => $val[3]);
        }
    }
    return $retVal;
} # end get_phpinfo
## end PmWikiInfo